#!/usr/bin/env python

import os, string, subprocess, sys

NATIVELIB     = 'nativelib.js'
NATIVELIB_DEV = 'nativelib-dev.js'

def build_script():
    minify = False
    if os.path.exists(os.path.join('builder', 'js_min')):
        minify = True
    else:
        if os.popen('which gcc').read():
            os.popen('gcc -o %s %s' % (os.path.join('builder', 'js_min'),
                                       os.path.join('builder', 'js_min.c')))
            minify = True
        else:
            print 'no gcc, cannot minify'

    script = '\n'.join(open('%s.js' % os.path.join('src', f)).read()
                         for f in include_order())
    if minify:
        handle = subprocess.Popen(os.path.join('builder', 'js_min'),
                                  stdout=subprocess.PIPE,
                                  stdin=subprocess.PIPE)
        open(NATIVELIB_DEV, 'w').write(script)
        print NATIVELIB_DEV
        script = handle.communicate(script)[0]
        open(NATIVELIB, 'w').write(script)
        print NATIVELIB
    else:
        open(NATIVELIB_DEV, 'w').write(script)
        print NATIVELIB_DEV
        open(NATIVELIB, 'w').write(script)
        print NATIVELIB

def build_tests():
    files = include_order()
    tag = '<script type="text/javascript" src="%s"></script>'
    prefix = '\n' + ' ' * 8
    template = string.Template(open(os.path.join('builder', 'templates', 'test.html')).read())
    scripts = prefix.join([tag % NATIVELIB_DEV])
    tests = prefix.join(tag % os.path.join('tests', '%s.js' % script) for script in files)
    open('tests.html', 'w').write(template.substitute(scripts=scripts, tests=tests))
    print 'tests.html'

def build_docs(path='../jsdoc-toolkit'):
    print 'docs/'
    subprocess.call(['java', '-jar', '%s/jsrun.jar' % path, '%s/app/run.js' % path,
                     '-t=%s/templates/jsdoc' % path, '-d=docs',
                     'nativelib-dev.js'])

def main():
    build_script()
    build_tests()

    if '-d' in sys.argv:
        if '-p' in sys.argv:
            try:
                build_docs(sys.argv[sys.argv.index('-p') + 1].rtrim('/'))
            except IndexError:
                pass
        else:
            build_docs()

def include_order():
    return [line.strip() for line in open(os.path.join('builder', 'order')).readlines()]

if __name__ == '__main__':
    main()
