Nativelib.merge(Math, {
  /**
   * Calculates the average (mean) of a list of numbers.
   *
   * @name average
   * @methodOf Math
   * @param {Array/Arguments} list An array or argument list of numbers.
   * @return {Number} Their average.
   */
  average: function (list) {
    var nums;
    if (testArray(list)) {
       nums = list;
    } else {
      nums = Array.create(arguments);
    }
    return Math.sum(nums) / nums.length;
  },

  /**
   * Creates a range between two numbers.
   *
   * @name range
   * @methodOf Math
   * @param {Number} a The start of the range.
   * @param {Number} b The end of the range.
   * @return {Array} The range.
   */
  range: function (a, b) {
    // Switch a and b so that the second is greater
    if (a > b) { var _b = b; b = a; a = _b; }
    var range = [];
    while (a <= b && range.push(a++)) {}
    return range;
  },

  /**
   * Calculates the sum of a list of numbers.
   *
   * @name sum
   * @methodOf Math
   * @param {Array/Arguments} list An array or argument list of numbers.
   * @return {Number} Their sum.
   */
  sum: function (list) {
    var nums;
    if (testArray(list)) {
      nums = list;
    } else {
      nums = Array.create(arguments);
    }
    return nums.reduce(function (a, b) {
      return a + b;
    });
  }
});

Nativelib.alias(Math, {
  /**
   * Alias of average.
   *
   * @name mean
   * @methodOf Math
   */
  average: 'mean'
});
