/**
 * @namespace Keeps track of nativelib-specific details.
 */
Nativelib = {
  record: {},

  /**
   * See Obj.merge.
   *
   * @name merge
   * @methodOf Nativelib
   */
  merge: function (o, props, clobber) {
    var merge;

    Nativelib.record[o] = Nativelib.record[o] || {};

    for (var p in props) {
      Nativelib.record[o][p] = merge = (props.hasOwnProperty(p) &&
        (clobber || typeof o[p] === 'undefined'));
      if (merge) {
        o[p] = props[p];
      }
    }
  },

  /**
   * See Obj.alias.
   *
   * @name alias
   * @methodOf Nativelib
   */
  alias: function (o, map) {
    var alias;

    Nativelib.record[o] = Nativelib.record[o] || {};

    for (var p in map) {
      if (map.hasOwnProperty(p)) {
        if (testArray(map[p])) {
          for (var i = 0, len = map[p].length; i < len; i++) {
            Nativelib.record[o][map[p][i]] = alias = typeof o[map[p][i]] === 'undefined';
            if (alias) {
              o[mao[p][i]] = o[p];
            }
          }
        } else {
          if (typeof o[map[p]] === 'undefined') {
            o[map[p]] = o[p];
          }
        }
      }
    }

    return o;
  },

  /**
   * Check if Nativelib wrote a given property on an object. Use it for
   * some of the more contentious properties that you find.
   *
   * TODO: Global clobber option so that all functions are applied.
   *
   * Example:
   *
   *   // true if Nativelib implemented invoke.
   *   Nativelib.wrote(Array.prototype, 'invoke');
   *
   * @name wrote
   * @methodOf Nativelib
   * @param {Object} o The object to check.
   * @param {String} p The property to check.
   */
  wrote: function (o, p) {
    if (Nativelib.record[o]) {
      return Boolean(Nativelib.record[o][p]);
    }
    return false;
  }
};
