Nativelib.merge(Number.prototype, {
  /**
   * Calls the passed function 'n' times, where 'n' is this Number.
   *
   * @name times
   * @methodOf Number#
   * @param {Function} fn The function to call.
   * @param {Object} ctx The execution context for the function.
   */
  times: function (fn, ctx) {
    for (var i = 1; i <= this; i++) {
      fn.call(ctx || this, i);
    }
  }
});

// Make all Math methods callable from a Number
Nativelib.merge(Number.prototype, Obj.create(
  ['abs', 'acos', 'asin', 'atan', 'atan2', 'ceil', 'cos', 'exp',
    'floor', 'log', 'max', 'min', 'pow', 'random', 'round', 'sin',
    'sqrt', 'tan'
  ].concat(
    Obj.keys(Math)
  ).map(function (fn) {
    return [fn, function () {
      return Math[fn].apply(this, [this].concat(
        Array.prototype.slice.apply(arguments)
      ));
    }];
  })
));

Nativelib.merge(Number, {
  /**
   * Checks if the passed object is a Number.
   *
   * @name test
   * @methodOf Number
   * @param {Object} o The object to check.
   * @return {Boolean} true if it is a number, false if not.
   */
  test: function (o) {
    return typeof o === 'number';
  }
});

Nativelib.alias(Number.prototype, {
  /**
   * Alias of range.
   *
   * @name to
   * @methodOf Number#
   */
  range: 'to'
});
