$(function () {
  module('Array');

  var simple = [1, 2, 3];
  var args = (function () {
    return arguments;
  }());

  test('all', function () {
    ok(simple.all(function (x) {
      return Number.test(x);
    }));
    ok(!simple.all(function (x) {
      return x < 2;
    }));
  });

  test('copy', function () {
    var cp = simple.copy();
    ok(simple.equals(cp) && cp !== simple, 'An array is copied.');
  });

  test('create', function () {
    ok(Array.test(Array.create(args)), 'Arguments becomes Array');
    ok(Array.test(Array.create('An Array')), 'A String becomes an Array');
    ok(Array.create([1])[0] === 1, 'An Array remains itself');
  });

  test('eachProperty', function () {
    var obj = { a: 1, b: 2, c: 3, d: function () { return 4; } };
    var arr = [obj, obj, obj];
    ok(arr.pluck('b').equals([2, 2, 2]), 'Each property is returned.');
    ok(arr.pluck('d', true).equals([4, 4, 4]), 'Each function is called.');
  });

  test('equals', function () {
    ok(simple.equals([1, 2, 3]), 'Equal arrays are equal.');
    ok(!simple.equals([3, 2, 1]), 'Unequal arrays are not equal.');
    ok(![2, 4, 8].equals([2, 4, 8, 10]), 'Similar arrays are not equal.');
    ok(![2, 4, 8, 10].equals([2, 4, 8]), 'Similar arrays are not equal.');
  });

  test('filter', function () {
    ok([-2, 0, 2].filter(function (n) {
      return n > 0;
    }).equals([2]), 'Filtering functions work as expected.');
  });

  test('filterOne', function () {
    ok([-3, -2, 10, 4, -10].filterOne(function (n) {
      return n > 0;
    }) === 10, 'The first match is filtered');
  });

  test('first', function () {
    ok([2, 3, 4].first() === 2, 'Returns the first item.');
    ok([].first() === undefined, 'undefined if no items');
  });

  test('flatten', function () {
    ok([[1], [2], [3]].flatten().equals([1, 2, 3]));
    ok([[1], [2, [8, 9]], [3]].flatten().equals([1, 2, 8, 9, 3]));
  });

  test('forEach', function () {
    var sum = 0;
    simple.forEach(function (n) {
      sum += n;
    });
    ok(sum === 6, 'Items are iterated predictably');
  });

  test('includes', function () {
    ok(simple.includes(1), 'Included number is found.');
    ok([2, false, 2].includes(false), 'Included boolean false is found.');
  });

  test('init', function () {
    ok(simple.init().equals([1, 2]), 'All but the last item is returned.');
    ok([].init().equals([]), 'An empty array returns an empty array.');
  });

  test('invoke', function () {
    var o = {
      a: function (x) {
        return x || 1;
      }
    };

    var a = (1).range(10).map(function () {
      return Obj.copy(o);
    });
    var b = (1).range(10).map(function () {
      return 1;
    });
    var c = (1).range(10).map(function () {
      return 2;
    });

    ok(a.invoke('a').equals(b));
    ok(a.invoke('a', [2]).equals(c));
  });

  test('last', function () {
    ok(simple.last() === 3, 'The last item is returned.');
    ok([].last() === undefined, 'An empty list returns undefined.');
  });

  test('map', function () {
    ok(simple.map(function (n) {
      return n * 2;
    }).equals([2, 4, 6]), 'A map function is applied predictably.');
  });

  test('partition', function () {
    var parts = [-2, -1, 0, 1, 2].partition(function (n) {
      return n > 0
    });
    ok(
      parts[0].equals([1, 2]) && parts[1].equals([-2, -1, 0]),
      'A simple partition function splits the array as expected.'
    );
  });

  test('reduce', function () {
    ok(simple.reduce(function (a, b) {
      return a + b;
    }) === 6, 'A reduction is preformed predictably.');
    ok(simple.reduce(function (a, b) {
      return a + b;
    }, 10) === 16, 'An initial value is included.');
  });

  test('remove', function () {
    var tmp = [2, 3, 2, 8];
    tmp.remove(2);
    ok(tmp.equals([3, 8]), 'All matches are removed.');
    ok(tmp.remove(3) === true, 'true is returned for a match.');
    ok(tmp.remove(0) === false, 'false is returned for a miss.');
  });

  test('some', function () {
    ok(simple.some(function (x) {
      return x === 1;
    }));
    ok(!simple.some(function (x) {
      return x < 1;
    }));
  });

  test('subtract', function () {
    var o = { 'a': 2 };

    ok([1, 2].equals([1, 2, 3].subtract([3])));
    ok([].equals([o, o, o].subtract([o])));
    ok([1].equals([1, '0'].subtract([0], true)));
  });

  test('test', function () {
    ok(Array.test([]), 'Array is true');
    ok(!Array.test({}), 'Object is false');
    ok(!Array.test(''), 'String is false');
    ok(!Array.test(args), 'Arguments are false');
  });

  test('unique', function () {
    ok([1, 2, 3, 1].unique().equals(simple), 'Duplicate numbers are removed.');
  });

  test('zip', function () {
    ok(
      simple.zip([4, 5, 6]).equals([[1, 4], [2, 5], [3, 6]]),
      'Same-lengthed arrays zip accurately'
    );
    ok(
      simple.zip([5, 6]).equals([[1, 5], [2, 6], [3, undefined]]),
      'Different-lengthed arrays zip accurately'
    );
    ok(
      [4, 5].zip(simple).equals([[4, 1], [5, 2], [undefined, 3]]),
      'Different-lengthed arrays zip accurately'
    );
  });
});
