$(function() {
  module('Obj');

  var simple = { a: 1, b: 2, c: 3 };

  test('alias', function() {
    ok(Obj.alias({ a: 1 }, { a: 'b' }).b === 1, 'A property is aliased');
  });

  test('copy', function() {
    var cp = Obj.copy(simple);
    ok(
      Obj.equals(simple, cp) && simple !== cp,
      'The copied object is equal to its source.'
    );
  });

  test('create', function() {
    ok(Obj.equals(Obj.create([['a', 1], ['b', 2]]), {
      a: 1, b: 2
    }), 'An object is created to spec');
  });

  test('equals', function() {
    ok(
      Obj.equals({ a: 1, b: 2 }, { a: 1, b: 2 }),
      'Equal objects equate'
    );

    ok(
      !Obj.equals({ a: 1, b: 2, c: 2 }, { a: 1, b: 2 }),
      'Unequal objects do not equate'
    );
  });

  test('forEach', function() {
    var sum = 0, letters = '';
    Obj.forEach(simple, function(v, p) {
      sum += v;
      letters += p;
    });
    ok(sum === 6, 'Values are iterated predictably.');
    ok(letters === 'abc', 'Properties are iterated predictably.');
  });

  test('filter', function() {
    ok(Obj.equals(Obj.filter(simple, function(v) {
      return v > 2;
    }), { c: 3 }), 'Values are filtered predictably.');
  });

  test('filterOne', function() {
    ok(Obj.filterOne({ b: -2, c: 2 }, function(n) {
      return n > 0;
    }).c === 2, 'The first match is found.');
  });

  test('invoke', function() {
    var ret = 1;
    var ret2 = 2;
    var o = {
      a: function(x) {
        return x || ret;
      }
    };

    var keys = ['a', 'b', 'c'];

    var o = Obj.create(keys.map(function(char) {
      return [char, Obj.copy(o)];
    }));

    var o2 = Obj.create(keys.map(function(char) {
      return [char, ret];
    }));

    var o3 = Obj.create(keys.map(function(char) {
      return [char, ret2];
    }));

    ok(Obj.equals(o2, Obj.invoke(o, 'a')))
    ok(Obj.equals(o3, Obj.invoke(o, 'a', [ret2])))
  });

  test('keys', function() {
    ok(
      Obj.keys({ a: 1, b: 2 }).sort().equals(['a', 'b']),
      'Keys are the same'
    );
    ok(Obj.keys({}).length === 0, 'Empty returns no keys');
  });

  test('map', function() {
    ok(Obj.equals(Obj.map(simple, function(v) {
      return 2 * v;
    }), { a: 2, b: 4, c: 6 }), 'A mapping function is applied predictably.');
  });

  test('merge', function() {
    ok(Obj.merge({}, { a: 1 }).a === 1, 'Properties are copied');
    ok(
      Obj.merge({ a: 2 }, { a: 3 }).a === 2,
      'Existing properties are not clobbered'
    );
    ok(
      Obj.merge({ a: 2 }, { a: 3 }, true).a === 3,
      'Existing properties clobbered on demand'
    );
  });

  test('test', function() {
    ok(!Obj.test(1), 'A number is not an object.');
    ok(!Obj.test('[object Function]'), 'A string is not an object.');
    ok(!Obj.test(/a/), 'A RegExp is not an object.');
    ok(!Obj.test(new Date()), 'A Date is not an object.');
    ok(!Obj.test(function() {}), 'A function literal is not an object');
    ok(!Obj.test(Function()), 'A Function instance is not an object');
    ok(Obj.test({}), 'An object is an object.');
  });

  test('values', function() {
    ok(Obj.values(simple).sort().equals([1, 2, 3]), 'Values are returned.');
  });

  test('valuesEqual', function() {
    ok(Obj.valuesEqual(2, 2), 'Numbers are equal.');
    ok(Obj.valuesEqual('a', 'a'), 'Strings are equal.');
    ok(Obj.valuesEqual([1, 2], [1, 2]), 'Simple arrays are equal.');
    ok(Obj.valuesEqual({ a: 1 }, { a: 1 }), 'Simple objects are equal.');
    ok(
      !Obj.valuesEqual([[1, 2], [3, 4]], [[], []]),
      'Unequal nested arrays are not equal.'
    );
    ok(
      Obj.valuesEqual([[1, 2], [3, 4]], [[1, 2], [3, 4]]),
      'Equal nested arrays are not equal.'
    );
  });
});
