$(function () {
  module('String');

  test('capitalize', function () {
    ok('hello'.capitalize() === 'Hello', 'A string is capitalized.');
  });

  test('endsWidth', function () {
    ok('abc'.endsWith('bc'), 'Ending characters are found.');
    ok(!'cbc'.endsWith('ab'), 'No match is found.');
  });

  test('empty', function () {
    ok(''.empty(), 'A zero-length string is empty.');
    ok('      \n  '.empty(), 'A string with only whitespace is empty.');
    ok(!'abc'.empty(), 'A string with content is not empty.');
  });

  test('format', function () {
    ok('{0} {1}'.format('a', 'b') === 'a b', 'Simple formatting is accurate.');
    ok(
      '{1}, {1} and {2}'.format('a', 'b', 'c') === 'b, b and c',
      'Argument position is accounted for.'
    );
  });

  test('mult', function () {
    ok('o'.mult(3) === 'ooo', 'A string is multiplied');
  });

  test('slugify', function () {
    ok('Hello There'.slugify() === 'hello-there', 'A string is slugified');
    ok('Once, there was a cat that     did cat things!'.slugify()
      === 'once-there-was-a-cat-that-did-cat-things', 'A string is slugified');
  });

  test('sprintf', function () {
    ok(
      'Hello, %s'.sprintf('cat') === 'Hello, cat',
      'String arguments are formatted as strings.'
    );
    var dt = new Date();
    ok(
      'Hello, %d'.sprintf(dt) === 'Hello, ' + Number(dt),
      'Number arguments are formatted as numbers.'
    );
    ok(
      'Hello, %d'.sprintf(3.2) === 'Hello, 3',
      'Floating point numbers are rounded down for %d'
    );
  });

  test('startsWith', function () {
    ok('abc'.startsWith('ab'), 'Starting characters are found.');
    ok(!'cbc'.startsWith('ab'), 'No match is found.');
  });

  test('strip', function () {
    ok('   a   '.strip() === 'a', 'Whitespace is stripped from both sides.');
    ok('b'.strip() === 'b', 'Nothing is removed if no whitespace is present.');
  });

  test('test', function () {
    ok(String.test('[object Function]'), 'A string is a string.');
    ok(!String.test(1), 'A number is not a string.');
    ok(!String.test(/a/), 'A RegExp is not a string.');
    ok(!String.test(new Date()), 'A Date is not a string.');
    ok(!String.test({}), 'An object is not a string.');
    ok(!String.test(function () {}), 'A function literal is not a string.');
    ok(!String.test(Function ()), 'A function instance is not a string.');
  });

  test('words', function () {
    ok('a b c'.words().equals(['a', 'b', 'c']), 'A simple word list is parsed.');
    ok(
      'a  b   c'.words().equals(['a', 'b', 'c']),
      'All separating whitespace is removed'
    );
  });
});
